@@ -11,7 +11,15 @@ import gzip
 import seaborn as sns
 
 
-def vcf_to_df(path, additional_cols=[], **kwargs):
+def vcf_to_df(
+    path,
+    additional_cols=[],
+    additional_filters=[],
+    parse_filter=False,
+    drop_null=False,
+    force_keep=[],
+    **kwargs,
+):
     """
     transforms a vcf file into a dataframe file as best as it can
 
@@ -37,12 +45,34 @@ def vcf_to_df(path, additional_cols=[], **kwargs):
         "ReverseComplementedAlleles",
     ] + additional_cols
 
+    filters = [
+        "PASS",
+        "base_qual",
+        "clustered_events",
+        "fragment",
+        "germline",
+        "haplotype",
+        "map_qual",
+        "multiallelic",
+        "panel_of_normals",
+        "position",
+        "slippage",
+        "strand_bias",
+        "weak_evidence",
+    ] + additional_filters
+
+    FUNCO_DESC = "Functional annotation from the Funcotator tool."
+
+    dropped_cols = []
+
     def read_comments(f):
-        fields = {}
         description = {}
+        colnames = []
+        rows = 0
         for l in f:
             l = l.decode("utf-8") if type(l) is not str else l
             if l.startswith("##"):
+                rows += 1
                 if "FORMAT" in l[:20]:
                     res = l.split("ID=")[1].split(",")[0]
                     desc = l.split("Description=")[1][:-2]
@@ -52,60 +82,100 @@ def vcf_to_df(path, additional_cols=[], **kwargs):
                     if res == "FUNCOTATION":
                         print("parsing funcotator special")
                         for val in l.split("Description=")[1][:-2].split("|"):
-                            fields.update({val: []})
-                            description.update({val: "Funcotaror: " + val})
+                            val = val.split("Funcotation fields are: ")[-1]
+                            description.update({val: FUNCO_DESC})
                     else:
                         desc = l.split("Description=")[1][:-2]
                         description.update({res: desc})
-                        fields.update({res: []})
             elif l.startswith("#"):
                 colnames = l[1:-1].split("\t")
+                rows += 1
             else:
                 break
-        return fields, description, colnames
+        return description, colnames, rows
 
     if path.endswith(".gz"):
         with gzip.open(path, "r") as f:
-            fields, description, colnames = read_comments(f)
+            description, colnames, nrows_toskip = read_comments(f)
     else:
         with open(path, "r") as f:
-            fields, description, colnames = read_comments(f)
-    colnames = [i.lower() for i in colnames]
+            description, colnames, nrows_toskip = read_comments(f)
+    colnames = [i for i in colnames]
     csvkwargs = {
         "sep": "\t",
         "index_col": False,
         "header": None,
         "names": colnames,
-        "comment": "#",
+        "skiprows": nrows_toskip + kwargs.get("skiprows", 0),
     }
-    data = pd.read_csv(path, **csvkwargs)
+    data = pd.read_csv(path, **{**kwargs, **csvkwargs})
     print(description)
+    funco_fields = [k for k, v in description.items() if FUNCO_DESC in v]
+    fields = {k: [] for k, _ in description.items()}
     try:
-        for j, val in enumerate(data["info"].str.split(";").values.tolist()):
-            for v in val:
-                if v in uniqueargs:
-                    res.update({v: True})  
-                elif:
-
-                elif '=' in v:
-                    tuple(v.split("="))
+        for j, info in enumerate(data["INFO"].str.split(";").values.tolist()):
+            res = {}
+            # show speed
+            if j % 10_000 == 0:
+                print(j, end="\r")
+            for annot in info:
+                if annot in uniqueargs:
+                    res.update({annot: True})
+                elif "=" in annot:
+                    # taking care of the funcotator special fields
+                    if "FUNCOTATION" in annot:
+                        # for multi allelic site:
+                        annot = annot.replace("FUNCOTATION=", "")[1:-1]
+                        res.update({name: [] for name in funco_fields})
+                        for site in annot.split("],["):
+                            site = (
+                                site.replace("_%7C_", " ")
+                                .replace("_%20_", " ")
+                                .replace("_%2C_", ",")
+                                .replace("_%3D_", "=")
+                                .split("|")
+                            )
+                            for i, sub_annot in enumerate(site):
+                                res[funco_fields[i]].append(sub_annot)
+                        for k in funco_fields:
+                            res[k] = ",".join(res[k])
+                    else:
+                        k, annot = annot.split("=")
+                        res.update({k: annot})
                 else:
-                    raise ValueError("unknown argument: " + v)
-            for k in fields.keys():
+                    raise ValueError("unknown argument: " + annot)
+            for k in list(fields.keys()):
                 fields[k].append(res.get(k, None))
     except ValueError:
-        print(val)
+        print(annot)
         raise ValueError("unknown field")
-    except:
-        import pdb
 
-        pdb.set_trace()
     data = pd.concat(
-        [data.drop(columns="info"), pd.DataFrame(data=fields, index=data.index)], axis=1
+        [data.drop(columns="INFO"), pd.DataFrame(data=fields, index=data.index)], axis=1
     )
-    samples = colnames[9:]
+    if drop_null:
+        cols_to_drop = []
+        for f in funco_fields:
+            # drop columns that have the same value across all rows
+            uniq = data[f].unique()
+            if len(uniq) == 1 and f.lower() not in force_keep:
+                cols_to_drop.append(f)
+                continue
+            elif len(uniq) < 10:
+                # checking multi allelic stuff
+                multi = []
+                for v in uniq:
+                    multi += v.split(",")
+                if len(set(multi)) == 1 and f.lower() not in force_keep:
+                    cols_to_drop.append(f)
+        print("dropping uninformative columns:", cols_to_drop)
+        data = data.drop(columns=cols_to_drop)
+        dropped_cols += cols_to_drop
+    data.columns = [i.lower() for i in data.columns]
+    samples = [i.lower() for i in colnames[9:]]
+    print("\nthe samples are:", samples)
+    sorting = data["format"][0].split(":")
     for sample in samples:
-        sorting = data["format"][0].split(":")
         res = data[sample].str.split(":").values.tolist()
         maxcols = max([len(v) for v in res])
         if maxcols - len(sorting) > 0:
@@ -120,7 +190,45 @@ def vcf_to_df(path, additional_cols=[], **kwargs):
             ],
             axis=1,
         )
-    return data.drop(columns="format"), description
+
+    # subsetting filters
+    if parse_filter:
+        data[filters] = False
+        for f in filters:
+            data.loc[data["filter"].str.contains(f), f] = True
+        data = data.drop(columns="filter")
+        dropped_cols.append("filter")
+
+    # cleaning empty cols
+    data = data.drop(columns="format")
+    dropped_cols.append("format")
+
+    todrop = []
+    for val in ["gt", "ad", "af", "dp", "f1r2", "f2r1", "fad", "sb"]:
+        if val in data.columns.tolist():
+            todrop.append(val)
+    data = data.drop(columns=todrop)
+
+    if drop_null:
+        empty = data.columns[data.isna().sum() == len(data)].tolist()
+        empty = list(set(empty) - set(force_keep))
+        print("dropping empty columns:", empty)
+        data = data.drop(columns=empty)
+        dropped_cols += empty
+
+    # weird bug sometimes
+    if "SB_1" in data.columns.tolist():
+        loc = ~data.SB_1.isna()
+        data.loc[loc, "PGT"] = data.loc[loc, "SB"]
+        data.loc[loc, "SB"] = data.loc[loc, "SB_1_2_3"]
+        data = data.drop(columns=["SB_1", "SB_1_2_3"])
+        data = data.rename(columns={"SB_1_2": "PS", "SB_1": "PID"})
+    else:
+        loc = data.SB.isna()
+        data.loc[loc, "SB"] = data.loc[loc, "PGT"]
+        data.loc[loc, "PGT"] = None
+    # sorting out issue with
+    return data, description, dropped_cols
 
 
 def mafToMat(
